#include "stdafx.h"
#include "Complex.h"

void ComplexInit(COMPLEX* Dst, double Re, double Im)
{
	double aa = sqrt(Re*Re + Im*Im);
	if (aa > 0)
	{
		Dst->A = aa;
		Dst->NRe = Re / Dst->A;
		Dst->NIm = Im / Dst->A;
	}
	else
	{
		Dst->A = 0;
		Dst->NRe = 0;
		Dst->NIm = 0;
	}
}

void ComplexRandom(COMPLEX* Dst, double MinRe, double MaxRe, double MinIm, double MaxIm)
{
	double Re = ((MaxRe - MinRe)*(double)rand() / (double)RAND_MAX) + MinRe;
	double Im = ((MaxIm - MinIm)*(double)rand() / (double)RAND_MAX) + MinIm;
	ComplexInit(Dst, Re, Im);
}

double ComplexRe(COMPLEX* z)
{
	return z->A*z->NRe;
}

double ComplexIm(COMPLEX* z)
{
	return z->A*z->NIm;
}

void ComplexAdd(COMPLEX* Dst, COMPLEX* A, COMPLEX* B)
{
	if (A->A <= 0)
	{
		*Dst = *B;
		return;
	}
	if (B->A <= 0)
	{
		*Dst = *A;
		return;
	}

	double Re = A->NRe*A->A + B->NRe*B->A;
	double Im = A->NIm*A->A + B->NIm*B->A;

	double aa = sqrt(Re*Re + Im*Im);
	if (aa <= 0)
	{
		Dst->A = 0;
		Dst->NRe = 0;
		Dst->NIm = 0;
		return;
	}

	Dst->A = aa;
	Dst->NRe = Re / aa;
	Dst->NIm = Im / aa;
}

void ComplexSub(COMPLEX* Dst, COMPLEX* Minuent, COMPLEX* Subtrahend)
{
	if (Minuent->A <= 0)
	{
		*Dst = *Subtrahend;
		return;
	}
	if (Subtrahend->A <= 0)
	{
		*Dst = *Minuent;
		return;
	}

	double Re = Minuent->NRe*Minuent->A - Subtrahend->NRe*Subtrahend->A;
	double Im = Minuent->NIm*Minuent->A - Subtrahend->NIm*Subtrahend->A;

	double aa = sqrt(Re*Re + Im*Im);
	if (aa <= 0)
	{
		Dst->A = 0;
		Dst->NRe = 0;
		Dst->NIm = 0;
		return;
	}

	Dst->A = aa;
	Dst->NRe = Re / aa;
	Dst->NIm = Im / aa;
}

void ComplexMul(COMPLEX* Dst, COMPLEX* A, COMPLEX* B)
{
	Dst->A = A->A*B->A;
	double Re = A->NRe*B->NRe - A->NIm*B->NIm;
	double Im = A->NRe*B->NIm + A->NIm*B->NRe;
	//  Re  Im !!!
	double a = sqrt(Re*Re + Im*Im);
	if (a > DBL_EPSILON)
	{
		Re /= a;
		Im /= a;
	}

	Dst->NRe = Re;
	Dst->NIm = Im;
}

void ComplexDiv(COMPLEX* Dst, COMPLEX* Num, COMPLEX* Denom)
{
	COMPLEX tmp;
	tmp.A = 1;
	tmp.NRe = Denom->NRe;
	tmp.NIm = -Denom->NIm;

	ComplexMul(Dst, Num, &tmp);

	Dst->A /= Denom->A;
}

void ComplexIntPow(COMPLEX* Dst, COMPLEX* Base, unsigned int Exponent)
{
	COMPLEX TheBase = *Base;

	Dst->A = 1;
	Dst->NRe = 1;
	Dst->NIm = 0;

	for (unsigned int i = 0; i < Exponent; i++)
	{
		ComplexMul(Dst, Dst, &TheBase);
	}
}