#include "stdafx.h"
#include "Complex.h"
#include "ComplexApprox.h"

void MatrixFromPoints(POLY_POINT Points[], unsigned int NumOfPoints, COMPLEX Matrix[], unsigned int MatrixSize, COMPLEX b[])
{
	for (unsigned int p = 0; p < MatrixSize; p++)
	{
		ComplexInit(&b[p], 0, 0);
		for (unsigned int i = 0; i < NumOfPoints; i++)
		{
			COMPLEX Z = Points[i].x;
			Z.NIm = -Z.NIm;
			ComplexIntPow(&Z, &Z, p);

			ComplexMul(&Z, &Z, &Points[i].y);
			ComplexAdd(&b[p], &b[p], &Z);
		}

		for (unsigned int q = 0; q < MatrixSize; q++)
		{
			if (q >= p)
			{
				COMPLEX Apq;
				ComplexInit(&Apq, 0, 0);

				for (unsigned int i = 0; i < NumOfPoints; i++)
				{
					/*if ((p == 0) && (q == 4) && (i == 9))
					{
					printf("");
					}*/
					double AbsPow = pow(Points[i].x.A, 2 * p);

					COMPLEX ZPow;
					ComplexIntPow(&ZPow, &Points[i].x, q - p);

					ZPow.A *= AbsPow;
					ComplexAdd(&Apq, &Apq, &ZPow);
				}

				Matrix[p*MatrixSize + q] = Apq;
			}
			else
			{
				Matrix[p*MatrixSize + q] = Matrix[q*MatrixSize + p];
				Matrix[p*MatrixSize + q].NIm = -Matrix[p*MatrixSize + q].NIm;
			}
		}
	}
}

void MatrixToPointers(COMPLEX Matrix[], unsigned int Size, COMPLEX* Pointers[])
{
	for (unsigned int i = 0; i < Size; i++)
	{
		Pointers[i] = Matrix + i*Size;
	}
}

double CalcDelta(COMPLEX Vec1[], COMPLEX Vec2[], unsigned int Size)
{
	double Result = 0;

	for (unsigned int i = 0; i < Size; i++)
	{
		COMPLEX tmp;
		ComplexSub(&tmp, &Vec1[i], &Vec2[i]);
		Result += tmp.A*tmp.A;
	}

	return sqrt(Result);
}

void Relax(COMPLEX* Pointers[], COMPLEX b[], unsigned int Size, double Epsilon, COMPLEX Result[], COMPLEX Prev[])
{
	unsigned int g = 0;

	//memcpy(Result, b, sizeof(COMPLEX)*Size);
	memset(Result, 0, sizeof(COMPLEX)*Size);
	do
	{
		memcpy(Prev, Result, sizeof(COMPLEX)*Size);

		for (unsigned int i = 0; i < Size; i++)
		{
			Result[i] = b[i];
			/*for (unsigned int j = 0; j < i; j++)
			{
				COMPLEX tmp;
				ComplexMul(&tmp, &Pointers[i][j], &Result[j]);
				ComplexSub(&Result[i], &Result[i], &tmp);
			}*/

			//for (unsigned int j = i; j < Size; j++)
			for (unsigned int j = 0; j < Size; j++)
			{
				COMPLEX tmp;
				ComplexMul(&tmp, &Pointers[i][j], &Prev[j]);
				ComplexSub(&Result[i], &Result[i], &tmp);
			}
			ComplexDiv(&Result[i], &Result[i], &Pointers[i][i]);
			//q=0.1
			Result[i].A *= 0.1;
			ComplexAdd(&Result[i], &Result[i], &Prev[i]);
		}

		g++;
		if (g % 1000 == 0)
		{
			printf("%10.10f\n", CalcDelta(Result, Prev, Size));
		}
	} while (CalcDelta(Result, Prev, Size) >= Epsilon);
}

void CalcPoly(COMPLEX Poly[], unsigned int Size, COMPLEX* z, COMPLEX* Result)
{
	*Result = Poly[Size - 1];
	for (unsigned int i = Size - 1; i > 0; i--)
	{
		unsigned int index = i - 1;
		ComplexMul(Result, Result, z);
		ComplexAdd(Result, Result, &Poly[index]);
	}
}